package com.example.kotlinopengles32

import javax.microedition.khronos.egl.EGLConfig
import javax.microedition.khronos.opengles.GL10
import android.opengl.GLES32
import android.opengl.GLSurfaceView
import android.opengl.Matrix
import android.content.Context
import android.graphics.Point
import android.view.MotionEvent
import cyberdeliaengine.*

class MyGLRenderer(context: Context) : GLSurfaceView.Renderer {
    private var mModel: OrangeGirl? = null
    private var mModel2: PantsGirl? = null

    private val mProjectionMatrix = FloatArray(16)
    private val mViewMatrix = FloatArray(16)
    var mPoint = Point(0, 0)
    private var mContext: Context = context
    private var mTouch: Boolean = false
    private var mHeight: Int = 0

    init {
    }

    override fun onSurfaceCreated(unused: GL10, config: EGLConfig) {

        GLES32.glEnable(GLES32.GL_DEPTH_TEST)

        GLES32.glClearColor(0.4f, 0.7f, 1.0f, 1.0f)

        mModel = OrangeGirl()
        mModel?.setTexture(mContext.resources, R.drawable.checkorange)
        mModel?.playAnimation(0, true)
        mModel2 = PantsGirl()
        mModel2!!.setTexture(mContext.resources, R.drawable.pantsbrown)
        mModel2?.playAnimation(0, true)
    }

    override fun onDrawFrame(unused: GL10) {

        GLES32.glClear(GLES32.GL_COLOR_BUFFER_BIT or GLES32.GL_DEPTH_BUFFER_BIT)

        Matrix.setLookAtM(mViewMatrix, 0, 0f, 300f, 500f, 0f, 100f, 0f, 0f, 1.0f, 0.0f)

        mModel!!.transform(mViewMatrix, mProjectionMatrix)
        mModel!!.mTranslation.set(-100f, 0f, 0f)
        mModel!!.draw()

        mModel2!!.transform(mViewMatrix, mProjectionMatrix)
        mModel2!!.mTranslation.set(100f, 0f, 0f)
        mModel2!!.draw()
    }

    override fun onSurfaceChanged(unused: GL10, width: Int, height: Int) {
        GLES32.glViewport(0, 0, width, height)
        mHeight = mPoint.y - height
        val ratio = width.toFloat() / height

        perspective(60.toFloat(), ratio, 1.toFloat(), 10000.0.toFloat())
    }

    private fun perspective(fov: Float, aspect: Float, near: Float, far: Float) {
        val top = near * Math.tan(fov * Math.PI / 360.0).toFloat()
        val right = top * aspect
        Matrix.frustumM(mProjectionMatrix, 0, -right, right, -top, top, near, far)
    }

    fun onTouchEvent(event: MotionEvent): Boolean {
//        val x = event!!.x
//        val y = event!!.y - mHeight
//        val height: Int = mPoint.y - mHeight
        when (event.getAction()) {
            MotionEvent.ACTION_DOWN -> {
                mTouch = true
//                if (mModel?.mousePick(x, y, mPoint.x, height, mProjectionMatrix)!!) {
//                    mModel?.stopAnimation()
//                }
//                if (mModel2?.mousePick(x, y, mPoint.x, height, mProjectionMatrix)!!) {
//                    mModel2?.stopAnimation()
//                }
            }
            MotionEvent.ACTION_UP -> {
                mTouch = false
//                mModel?.playAnimation(0, true)
//                mModel2?.playAnimation(0, true)
            }
            MotionEvent.ACTION_MOVE -> {
                mModel?.mAngle = event.x
                mModel2?.mAngle = event.x
            }
        }
        return true
    }
}
