package cyberdeliaengine

import java.nio.ByteBuffer
import java.nio.ByteOrder
import java.nio.FloatBuffer
import android.opengl.GLES32

open class BoneModel3D : Model3D() {

    private val mAnimations =  ArrayList<ArrayList<ArrayList<Float>>>(0)
    private var mTime : Int = 0
    private var mCount: Float = 0f
    private var mPlaying = true
    private var mLoop = true

    init {
    }

    fun addB(x:Double,y:Double,z:Double,b:Int){
        mCoords.add(x.toFloat())
        mCoords.add(y.toFloat())
        mCoords.add(z.toFloat())
        mBones.add(b.toFloat())
    }

    fun addA(m0:Double,m1:Double,m2:Double,m3:Double,
             m4:Double,m5:Double,m6:Double,m7:Double,
             m8:Double,m9:Double,m10:Double,m11:Double,
             m12:Double,m13:Double,m14:Double,m15:Double) {
        val anim2:ArrayList<ArrayList<Float>> = mAnimations[mCurrentAnimation]
        val anim:ArrayList<Float> = anim2[mTime]
        anim.add(m0.toFloat())
        anim.add(m1.toFloat())
        anim.add(m2.toFloat())
        anim.add(m3.toFloat())
        anim.add(m4.toFloat())
        anim.add(m5.toFloat())
        anim.add(m6.toFloat())
        anim.add(m7.toFloat())
        anim.add(m8.toFloat())
        anim.add(m9.toFloat())
        anim.add(m10.toFloat())
        anim.add(m11.toFloat())
        anim.add(m12.toFloat())
        anim.add(m13.toFloat())
        anim.add(m14.toFloat())
        anim.add(m15.toFloat())
    }

    fun addA2() {
        val anim = ArrayList<Float>(0)
        mAnimations[mCurrentAnimation].add(anim)
        mTime = mAnimations[mCurrentAnimation].size - 1
    }

    fun addA3() {
        val anim = ArrayList<ArrayList<Float>>(0)
        mAnimations.add(anim)
        mCurrentAnimation = mAnimations.size - 1
    }

    override fun newModel3D() {
        val coords = FloatArray(mIndices.size*3)
        for (i in mIndices.indices) {
            val v = mIndices[i]*3
            coords[i*3] = mCoords[v]
            coords[i*3+1] = mCoords[v+1]
            coords[i*3+2] = mCoords[v+2]
        }
        val indices = ShortArray(mIndices.size)
        for (i in mIndices.indices) {
            indices[i] = i.toShort()
        }
        val uvs = FloatArray(mUVs.size)
        for (i in mUVs.indices) {
            uvs[i] = mUVs[i]
        }
        val bones = FloatArray(mIndices.size)
        for (i in mIndices.indices) {
            bones[i] = mBones[mIndices[i].toInt()]
        }

        val bb = ByteBuffer.allocateDirect(coords.size * mFloatSizeBytes)
        bb.order(ByteOrder.nativeOrder())
        mVertexBuffer = bb.asFloatBuffer()
        mVertexBuffer!!.put(coords)
        mVertexBuffer!!.position(0)

        val bbb = ByteBuffer.allocateDirect(bones.size * mFloatSizeBytes)
        bbb.order(ByteOrder.nativeOrder())
        mBoneBuffer = bbb.asFloatBuffer()
        mBoneBuffer!!.put(bones)
        mBoneBuffer!!.position(0)

        val uvbb = ByteBuffer.allocateDirect(
              mUVs.size * mFloatSizeBytes)
        uvbb.order(ByteOrder.nativeOrder())
        mTextureBuffer = uvbb.asFloatBuffer()
        for (i in mUVs.indices) {
          mTextureBuffer!!.put(mUVs[i])
        }
        mTextureBuffer!!.position(0)

        val vertexShader = loadShader(
         GLES32.GL_VERTEX_SHADER,
         mVertexShaderCode)
        val fragmentShader = loadShader(
         GLES32.GL_FRAGMENT_SHADER,
         mFragmentShaderCode)

        mProgram = GLES32.glCreateProgram()
        GLES32.glAttachShader(mProgram, vertexShader)
        GLES32.glAttachShader(mProgram, fragmentShader)
        GLES32.glLinkProgram(mProgram)
    }

    override fun draw() {
        if ( mPlaying )
        {
            mCount += 30f/60f
        }
        if ( mCount >= mAnimations[mCurrentAnimation].size ) {
            mCount = 0f
            if ( !mLoop ) mPlaying = false
        }
        val animations = FloatArray(mAnimations[mCurrentAnimation][mCount.toInt()].size)
        for (i in mAnimations[mCurrentAnimation][mCount.toInt()].indices) {
            animations[i] = mAnimations[mCurrentAnimation][mCount.toInt()][i]
        }

        GLES32.glUseProgram(mProgram)

        mPositionHandle = GLES32.glGetAttribLocation(mProgram, "a_Position")

        GLES32.glEnableVertexAttribArray(mPositionHandle)

        GLES32.glVertexAttribPointer(
                mPositionHandle, mCoordsPerVertex,
                GLES32.GL_FLOAT, false,
                mVertexStrideBytes, mVertexBuffer)

        mBoneHandle = GLES32.glGetAttribLocation(mProgram, "a_Bone")
        GLES32.glEnableVertexAttribArray(mBoneHandle)
        GLES32.glVertexAttribPointer(
                mBoneHandle, mCoordsPerVertex,
                GLES32.GL_FLOAT, false,
                mBoneStrideBytes, mBoneBuffer)

        GLES32.glActiveTexture(GLES32.GL_TEXTURE0)
        GLES32.glBindTexture(GLES32.GL_TEXTURE_2D, mTextureID)

        aTextureHandle = GLES32.glGetAttribLocation(mProgram, "a_TextureCoord")
        GLES32.glEnableVertexAttribArray(aTextureHandle)
        GLES32.glVertexAttribPointer(
         aTextureHandle, 2,
         GLES32.GL_FLOAT, false,
         mUvDataStrideBytes, mTextureBuffer)

        mMVPMatrixHandle = GLES32.glGetUniformLocation(mProgram, "u_MVPMatrix")
        GLES32.glUniformMatrix4fv(mMVPMatrixHandle, 1, false, mMVPMatrix, 0)

        uBoneMatrixHandle = GLES32.glGetUniformLocation(mProgram, "u_BoneMatrix")
        GLES32.glUniformMatrix4fv(
                uBoneMatrixHandle,
                mAnimations[mCurrentAnimation][mCount.toInt()].size/16,
                false,
                animations,
                0
        )

        mUseBoneHandle = GLES32.glGetUniformLocation(mProgram, "u_UseBone")
        GLES32.glUniform4f(mUseBoneHandle,1f,1f,1f,1f)

        GLES32.glDrawArrays(GLES32.GL_TRIANGLES,0, mIndices.size)

        GLES32.glDisableVertexAttribArray(mPositionHandle)
    }

    fun playAnimation(animation:Int,loop:Boolean)
    {
        mPlaying = true
        mCurrentAnimation = animation
        mLoop = loop
    }

    fun stopAnimation()
    {
        mPlaying = false
    }
}
